import os

import tempfile
import ConfigParser
from collections import OrderedDict

from linchpin.api import LinchpinError, LinchpinAPI

"""
Provide valid API data to test against.

"""

class APIData(object):

    def __init__(self):

        pass

